open Tags
open Syntax

let rec check_wf x seen = function
  | Seq (r1,r2) 
  | Test (_,r1,r2) -> check_wf x seen r1; check_wf x seen r2
  | Elem (_,r) | Star r -> check_wf x seen r
  | Name y when x = y -> failwith ("Non-linear: " ^ x)
  | Name y when List.mem y seen -> ()
  | Name y -> ignore (def y); check_wf x (y::seen) (def x)
  | _ -> ()

let name_checked = Hashtbl.create 64

let rec check = function
  | Eps -> ()
  | Sub c | Next c | Star c -> check c
  | Elem (s,r) ->
      if not (SymbolSet.is_singleton s
	      || SymbolSet.is_any s) then failwith "Not an expression";
      check r
  | Plus _ | Alt _ -> failwith "Not an expression"
  | Test (r,r1,r2) -> Regexp.check r; check r1; check r2
  | Seq (r1,r2) -> check r1; check r2
  | Name x when Hashtbl.mem name_checked x -> ()
  | Name x -> 
      let d = Syntax.def x in 
      check_wf x [] d; 
      Hashtbl.add name_checked x ();
      check d

