module Make(X : sig type t val compare: t -> t -> int end) = struct
  type t = X.t list

  let empty = []

  let is_empty l = l == []


  let rec union l1 l2 = match l1,l2 with
    | v1::q1, v2::q2 ->
	let c = X.compare v1 v2 in
	if c = 0 then v1::(union q1 q2)
	else if c < 0 then v1::(union q1 l2)
	else v2::(union l1 q2)
    | [], l | l, [] -> l

  let add x l = union [x] l

  let rec inter l1 l2 = match l1,l2 with
    | v1::q1, v2::q2 ->
	let c = X.compare v1 v2 in
	if c = 0 then v1::(inter q1 q2)
	else if c < 0 then inter q1 l2
	else inter l1 q2
    | [], l | l, [] -> []

  let rec diff l1 l2 = match l1,l2 with
    | v1::q1, v2::q2 ->
	let c = X.compare v1 v2 in
	if c = 0 then diff q1 q2
	else if c < 0 then v1::(diff q1 l2)
	else diff l1 q2
    | [], l -> [] | l, [] -> l

  let rec mem v l = match l with
    | v1::q1 -> let c = X.compare v1 v in (c = 0) || (c < 0 && mem v q1)
    | [] -> false

  let rec subset l1 l2 = match l1,l2 with
    | [], _ -> true
    | v1::q1, v2::q2 ->
	let c = X.compare v1 v2 in 
	(c = 0 && subset q1 q2) || (c > 0 && subset l1 q2)
    | _ -> false

  let rec compare l1 l2 = match l1,l2 with
    | [], [] -> 0
    | [], l -> -1
    | l, [] -> 1
    | v1::q1, v2::q2 ->
	let c = X.compare v1 v2 in
	if c = 0 then compare q1 q2 else c

  let iter = List.iter

  let cardinal = List.length

  let min_elt = List.hd

end
